/* $Id: utilgmx.c,v 1.1 2002/01/14 11:17:33 oliver Exp $
   everyday functions
   $Log: utilgmx.c,v $
   Revision 1.1  2002/01/14 11:17:33  oliver
   Utility routines (debugging, quicksort,...)

   Revision 1.2  2001/02/19 14:26:22  oliver
   moved dt_tpx to utilgmx

   Revision 1.1  2001/02/18 16:38:13  oliver
   Initial revision

*/

static char *SRCID_utilgmx_c = "$Id: utilgmx.c,v 1.1 2002/01/14 11:17:33 oliver Exp $";

#include "utilgmx.h"
#include "fatal.h"
#include "futil.h"
#include "maths.h"

void quicksort(atom_id v[], int left, int right)
{
  int i,last;

  if (left >= right)                    /* Do nothing if array contains */
    return;                             /* fewer than two elements      */
  aswap(v,left,(left+right)/2);         /* Move partition element       */
  last=left;                            /* to v[0]                      */
  for(i=left+1; (i<=right); i++)        /* partition                    */
    if (v[i] < v[left]) {
      last++;
      aswap(v,last,i);                  /* watch out for macro trick    */
    }
  aswap(v,left,last);                   /* restore partition element    */
  quicksort(v,left,last-1);
  quicksort(v,last+1,right);
}


void msg (const char *format, ...)
{
  va_list args;
  va_start(args, format);

  vfprintf (stderr, format, args);
  if (bDebugMode()) {
    vfprintf(debug, format, args);
  };

  va_end(args);
  return;
};

void dmsg (const char *format, ...)
{
  va_list args;
  va_start(args, format);

  if (bDebugMode()) {
    vfprintf(stderr, format, args);
    vfprintf(debug,  format, args);
  };

  va_end(args);
  return;
};
  

void dprintf (const char *format, ...)
{
  va_list args;
  va_start(args, format);
  
  if (bDebugMode()) {
    vfprintf(stderr, format, args);
  };
  va_end(args);
  return;
};

void dfprintf (const char *format, ...)
{
  /* write to debug file (see fatal.h) */
  va_list args;
  va_start(args, format);
  
  if (bDebugMode()) {
    vfprintf(debug, format, args);
  };
  va_end(args);
  return;
};


real ldist (rvec x, rvec p, rvec c) {
  /* calculate the (perpendicular) distance of point x from the line
     through c pointing along p 
  */
  real lambda;
  rvec d, u;

  /* u = x - c */
  rvec_sub (x, c, u);
  
  /* d = [(u.p)/p^2]p  - u */
  lambda = iprod (u, p) / iprod (p, p);
  svmul (lambda, p, d);
  rvec_dec (d, u);

  return norm (d);
};

real dt_tpx (char *fn) {
  int step, natoms;
  real t, lambda;
  t_inputrec  ir;  

  /* discard almost all info */
  read_tpx(fn,&step,&t,&lambda, &ir, NULL, &natoms, NULL, NULL, NULL, NULL);

  return ir.delta_t * ir.nstxtcout;
};

